"""Metadata Primitives catalog module.

Provides the query catalog loader and dataclasses for metadata introspection queries.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from .loader import (
    MetadataCatalog,
    MetadataCatalogError,
    MetadataQuery,
    load_metadata_catalog,
)

__all__ = [
    "MetadataCatalog",
    "MetadataCatalogError",
    "MetadataQuery",
    "load_metadata_catalog",
]
