"""GCP platform adapters.

This module provides adapters for Google Cloud Platform data services.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

from benchbox.platforms.gcp.dataproc_adapter import DataprocAdapter
from benchbox.platforms.gcp.dataproc_serverless_adapter import DataprocServerlessAdapter

__all__ = ["DataprocAdapter", "DataprocServerlessAdapter"]
