"""CoffeeShop DataFrame queries for Expression and Pandas families.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

# Import queries module to trigger registration
from benchbox.core.coffeeshop.dataframe_queries import queries as _queries  # noqa: F401
from benchbox.core.coffeeshop.dataframe_queries.registry import (
    COFFEESHOP_DATAFRAME_QUERIES,
    get_coffeeshop_query,
    list_coffeeshop_queries,
)

__all__ = [
    "COFFEESHOP_DATAFRAME_QUERIES",
    "get_coffeeshop_query",
    "list_coffeeshop_queries",
]
