"""TSBS DevOps DataFrame queries for Expression and Pandas families.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from __future__ import annotations

# Import queries module to trigger registration
from benchbox.core.tsbs_devops.dataframe_queries import queries as _queries  # noqa: F401
from benchbox.core.tsbs_devops.dataframe_queries.registry import (
    TSBS_DEVOPS_DATAFRAME_QUERIES,
    get_tsbs_devops_query,
    list_tsbs_devops_queries,
)

__all__ = [
    "TSBS_DEVOPS_DATAFRAME_QUERIES",
    "get_tsbs_devops_query",
    "list_tsbs_devops_queries",
]
