"""Join Order Benchmark core implementation.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from .benchmark import JoinOrderBenchmark

__all__ = ["JoinOrderBenchmark"]
