"""Core operation execution interfaces.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from benchbox.core.operations.base import OperationExecutor

__all__ = ["OperationExecutor"]
