"""Shared infrastructure for primitives benchmarks (write, transaction, read, metadata).

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""
