"""Modular TPC-DS data generation package."""

from .filesystem import FileArtifactMixin
from .manager import TPCDSDataGenerator
from .runner import DsdgenRunnerMixin
from .streaming import StreamingGenerationMixin

TPCDSGenerator = TPCDSDataGenerator

__all__ = [
    "TPCDSDataGenerator",
    "TPCDSGenerator",
    "DsdgenRunnerMixin",
    "StreamingGenerationMixin",
    "FileArtifactMixin",
]
