"""Reference-aligned CoffeeShop benchmark module."""

from benchbox.core.coffeeshop.benchmark import CoffeeShopBenchmark
from benchbox.core.coffeeshop.generator import CoffeeShopDataGenerator
from benchbox.core.coffeeshop.queries import CoffeeShopQueryManager

__all__ = ["CoffeeShopBenchmark", "CoffeeShopDataGenerator", "CoffeeShopQueryManager"]
