"""Shared catalog loading infrastructure for primitives benchmarks.

Copyright 2026 Joe Harris / BenchBox Project

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from benchbox.core.primitives.catalog.loader import (
    load_operations_catalog,
    load_query_catalog,
)

__all__ = [
    "load_operations_catalog",
    "load_query_catalog",
]
