"""Read Primitives benchmark module.

This module provides functionality to run primitive database read operations tests
based on the TPC-H schema. The Read Primitives benchmark includes queries that test
fundamental database read operations like aggregations, joins, filters, and more
advanced analytical operations.

Copyright 2026 Joe Harris / BenchBox Project

This implementation is derived from TPC Benchmark™ H (TPC-H) - Copyright © Transaction Processing Performance Council

Licensed under the MIT License. See LICENSE file in the project root for details.
"""

from .benchmark import ReadPrimitivesBenchmark
from .generator import ReadPrimitivesDataGenerator
from .queries import ReadPrimitivesQueryManager
from .schema import TABLES, get_all_create_table_sql

__all__ = [
    "ReadPrimitivesBenchmark",
    "ReadPrimitivesDataGenerator",
    "ReadPrimitivesQueryManager",
    "TABLES",
    "get_all_create_table_sql",
]
