"""Platform-specific query plan parsers."""

from benchbox.core.query_plans.parsers.base import QueryPlanParser
from benchbox.core.query_plans.parsers.datafusion import DataFusionQueryPlanParser
from benchbox.core.query_plans.parsers.duckdb import DuckDBQueryPlanParser
from benchbox.core.query_plans.parsers.postgresql import PostgreSQLQueryPlanParser
from benchbox.core.query_plans.parsers.redshift import RedshiftQueryPlanParser
from benchbox.core.query_plans.parsers.registry import (
    ParserRegistry,
    get_parser_for_platform,
    get_parser_registry,
    reset_global_registry,
)
from benchbox.core.query_plans.parsers.sqlite import SQLiteQueryPlanParser

__all__ = [
    "QueryPlanParser",
    "DataFusionQueryPlanParser",
    "DuckDBQueryPlanParser",
    "PostgreSQLQueryPlanParser",
    "RedshiftQueryPlanParser",
    "SQLiteQueryPlanParser",
    # Registry
    "ParserRegistry",
    "get_parser_registry",
    "get_parser_for_platform",
    "reset_global_registry",
]
