"""Variant registries for TPC-Havoc queries."""

from __future__ import annotations

from . import (
    q01,
    q02,
    q03,
    q04,
    q05,
    q06,
    q07,
    q08,
    q09,
    q10,
    q11,
    q12,
    q13,
    q14,
    q15,
    q16,
    q17,
    q18,
    q19,
    q20,
    q21,
    q22,
)

Q1_VARIANTS = q01.VARIANTS
Q2_VARIANTS = q02.VARIANTS
Q3_VARIANTS = q03.VARIANTS
Q4_VARIANTS = q04.VARIANTS
Q5_VARIANTS = q05.VARIANTS
Q6_VARIANTS = q06.VARIANTS
Q7_VARIANTS = q07.VARIANTS
Q8_VARIANTS = q08.VARIANTS
Q9_VARIANTS = q09.VARIANTS
Q10_VARIANTS = q10.VARIANTS
Q11_VARIANTS = q11.VARIANTS
Q12_VARIANTS = q12.VARIANTS
Q13_VARIANTS = q13.VARIANTS
Q14_VARIANTS = q14.VARIANTS
Q15_VARIANTS = q15.VARIANTS
Q16_VARIANTS = q16.VARIANTS
Q17_VARIANTS = q17.VARIANTS
Q18_VARIANTS = q18.VARIANTS
Q19_VARIANTS = q19.VARIANTS
Q20_VARIANTS = q20.VARIANTS
Q21_VARIANTS = q21.VARIANTS
Q22_VARIANTS = q22.VARIANTS

VARIANT_REGISTRY = {
    1: Q1_VARIANTS,
    2: Q2_VARIANTS,
    3: Q3_VARIANTS,
    4: Q4_VARIANTS,
    5: Q5_VARIANTS,
    6: Q6_VARIANTS,
    7: Q7_VARIANTS,
    8: Q8_VARIANTS,
    9: Q9_VARIANTS,
    10: Q10_VARIANTS,
    11: Q11_VARIANTS,
    12: Q12_VARIANTS,
    13: Q13_VARIANTS,
    14: Q14_VARIANTS,
    15: Q15_VARIANTS,
    16: Q16_VARIANTS,
    17: Q17_VARIANTS,
    18: Q18_VARIANTS,
    19: Q19_VARIANTS,
    20: Q20_VARIANTS,
    21: Q21_VARIANTS,
    22: Q22_VARIANTS,
}

__all__ = [
    "Q1_VARIANTS",
    "Q2_VARIANTS",
    "Q3_VARIANTS",
    "Q4_VARIANTS",
    "Q5_VARIANTS",
    "Q6_VARIANTS",
    "Q7_VARIANTS",
    "Q8_VARIANTS",
    "Q9_VARIANTS",
    "Q10_VARIANTS",
    "Q11_VARIANTS",
    "Q12_VARIANTS",
    "Q13_VARIANTS",
    "Q14_VARIANTS",
    "Q15_VARIANTS",
    "Q16_VARIANTS",
    "Q17_VARIANTS",
    "Q18_VARIANTS",
    "Q19_VARIANTS",
    "Q20_VARIANTS",
    "Q21_VARIANTS",
    "Q22_VARIANTS",
    "VARIANT_REGISTRY",
]
